unit roomsunit;

{$mode objfpc}{$H+}

interface

const
  // Konstante koje oznacavaju komande
  // Idi na sever
  CNorth = 1;
  // Idi na zapad
  CWest = 2;
  // Idi na jug
  CSouth = 3;
  // Idi na istok
  CEast = 4;

type
  // Podaci vezani za sobu
  TRoom = record
    // Opis sobe
    Decription: String;
    // Prolazi do drugih soba
    Doors: array[CNorth..CEast] of Integer;
  end;

  TRooms = array of TRoom;

// Ucitavanje soba iz fajla
function LoadRooms(FileName: String; var Rooms: TRooms): Integer; overload;
function LoadRooms(var HFile: File; var Rooms: TRooms): Integer; overload;

// Snimanje soba u fajl
function SaveRooms(FileName: String; var Rooms: TRooms; Count: Integer = 0): Integer; overload;
function SaveRooms(var HFile: File; var Rooms: TRooms; Count: Integer = 0): Integer; overload;

implementation

const
  // Header fajla u kojem se snimaju sobe (ROOM)
  CROOM_HEADER = $4D4F4F52;

// Ucitavanje soba iz fajla
function LoadRooms(FileName: String; var Rooms: TRooms): Integer;
var
  HFile: File;
begin
  // Pretpostavimo da ce doci do greske
  Result := -1;
  
  // Otvaramo fajl
  Assign(HFile, FileName);
  try
    Reset(HFile, 1);
    // Ucitaj iz fajla
    Result := LoadRooms(HFile, Rooms);
  finally
    // Zatvori fajl
    Close(HFile);
  end;
end;

function LoadRooms(var HFile: File; var Rooms: TRooms): Integer; overload;
var
  Header: Cardinal;
  // Broj fajlova procitanih iz fajla
  ReadCount: Cardinal;
  Count: Integer;
  I: Integer;
  Len: Integer;
begin
  // Pretpostavimo da ce doci do greske
  Result := -1;

  // Ucitavanje headera
  BlockRead(HFile, Header, SizeOf(Header), ReadCount);
  if (ReadCount <> SizeOf(Header)) or (Header <> CROOM_HEADER) then
    Exit;
    
  // Citamo broj soba
  BlockRead(HFile, Count, SizeOf(Count), ReadCount);
  if ReadCount <> SizeOf(Count) then
    Exit;
    
  // Postavljamo velicinu niza
  SetLength(Rooms, Count);
    
  // Ucitavanje soba
  for I := 0 to Count - 1 do
  begin
    // Citamo duzinu opisa
    BlockRead(HFile, Len, SizeOf(Len), ReadCount);
    if ReadCount <> SizeOf(Len) then
      Exit;

    // Postavljamo duzinu opisa
    SetLength(Rooms[I].Decription, Len);
    
    // Citamo opis sobe
    BlockRead(HFile, Rooms[I].Decription[1], Len, ReadCount);
    if ReadCount <> Len then
      Exit;

    // Citamo prolaze do drugih soba
    BlockRead(HFile, Rooms[I].Doors, SizeOf(Rooms[I].Doors), ReadCount);
    if ReadCount <> SizeOf(Rooms[I].Doors) then
      Exit;
  end;

  // Sve je uspesno zavrseno
  Result := Count;
end;

// Snimanje soba u fajl
function SaveRooms(FileName: String; var Rooms: TRooms; Count: Integer): Integer; overload;
var
  HFile: File;
begin
  // Pretpostavimo da ce doci do greske
  Result := -1;

  // Otvaramo fajl
  Assign(HFile, FileName);
  try
  Rewrite(HFile, 1);
  // Snimi u fajl
  Result := SaveRooms(HFile, Rooms, Count);
  finally
  // Zatvori fajl
  Close(HFile);
  end;
end;

function SaveRooms(var HFile: File; var Rooms: TRooms; Count: Integer): Integer; overload;
var
  // Broj bajtova upisanih u fajl
  WriteCount: Cardinal;
  I: Integer;
  Len: Integer;
begin
  // Pretpostavimo da ce doci do greske
  Result := -1;
  
  // Upisemo header
  BlockWrite(HFile, CROOM_HEADER, SizeOf(CROOM_HEADER), WriteCount);
  if WriteCount <> SizeOf(CROOM_HEADER) then
    Exit;
    
  // Uzimamo broj soba iz niza ako korisnik nije nista uneo
  if Count = 0 then
    Count := High(Rooms) + 1;
    
  // Upisujemo broj soba
  BlockWrite(HFile, Count, SizeOf(Count), WriteCount);
  if WriteCount <> SizeOf(Count) then
    Exit;

  // snimanje soba
  for I := 0 to Count - 1 do
  begin
    // Upisujemo duzinu opisa
    Len := Length(Rooms[I].Decription);
    BlockWrite(HFile, Len, SizeOf(Len), WriteCount);
    if WriteCount <> SizeOf(Len) then
      Exit;

    // Upisujemo opis sobe
    BlockWrite(HFile, Rooms[I].Decription[1], Len, WriteCount);
    if WriteCount <> Len then
      Exit;
      
    // Upisujemo prolaze do drugih soba
    BlockWrite(HFile, Rooms[I].Doors, SizeOf(Rooms[I].Doors), WriteCount);
    if WriteCount <> SizeOf(Rooms[I].Doors) then
      Exit;
  end;
  
  // Sve je uspesno zavrseno
  Result := Count;
end;

end.

