unit gameunit;

{$mode objfpc}{$H+}

interface

uses
  roomsunit;

var
  // Sve sobe koje kreiramo
  Rooms: TRooms;

// Glavni meni u editoru
function MainMenu: Integer;

// Biranje sobe za izmenu/brisanje
function SelectRoom: Integer;

// Dodavanje nove sobe
procedure AddRoom;

// Brisanje sobe
procedure RemoveRoom(RoomIndex: Integer);

// Menjanje sobe
procedure EditRoom(RoomIndex: Integer);

// Ucitaj sobe
procedure Load;

// Snimi sobe
procedure Save;

implementation

uses
  crt;

// Skracuje opis
function GetShortDescription(Desc: String; Count: Integer): String;
var
  I: Integer;
begin
  if Length(Desc) <= Count then
    // Uzimamo ceo string ako nije previse dug
    Result := Desc
  else
    // Uzimamo pocetak stringa i dodajemo ... na kraju
    Result := Copy(Desc, 1, Count - 3) + '...';

  // Znak za novi red menja u razmak da bi se opis lepse prikazao
  for I := 1 to Length(Result) do
    if Result[I] = #10 then
      Result[I] := ' ';
end;

// Glavni meni u editoru
function MainMenu: Integer;
begin
  ClrScr;
  WriteLn('Rooms Editor v1.0');
  WriteLn;
  
  WriteLn('1. Add room');
  WriteLn('2. Remove room');
  WriteLn('3. Edit room');
  WriteLn;
  WriteLn('4. Load rooms');
  WriteLn('5. Save rooms');
  WriteLn;
  WriteLn('0. Exit editor');
  WriteLn;

  repeat
    Write('Select menu: ');
    ReadLn(Result);
  until Result in [0..5];
end;

// Biranje sobe za izmenu/brisanje
function SelectRoom: Integer;
var
  I: Integer;
begin
  ClrScr;
  WriteLn('Select room');
  WriteLn;
  
  for I := 0 to High(Rooms) do
    WriteLn(I + 1, '. ', GetShortDescription(Rooms[I].Decription, 65));

  WriteLn;
  WriteLn('0. Cancel');
  WriteLn;

  repeat
    Write('Select menu: ');
    ReadLn(Result);
  until (Result >= 0) and (Result <= High(Rooms) + 1);
end;

// Dodavanje nove sobe
procedure AddRoom;
var
  Desc: String;
  Line: String;
begin
  ClrScr;
  WriteLn('Add room');
  WriteLn('Doors can be defined in Edit room mode');
  WriteLn;
  
  WriteLn('Enter description (to finish type <end> at beginig of line):');
  
  // Uzimamo opis
  Desc := '';
  Line := '';
  repeat
    Desc := #10 + Line;
    ReadLn(Line);
  until Line = '<end>';
  
  // Izbacujemo prvi #10
  Desc := Copy(Desc, 2, Length(Desc));
  
  // Povecavamo niz (kod nije optimizovan)
  SetLength(Rooms, High(Rooms) + 2);
  
  // Postavljamo podatke
  FillChar(Rooms[High(Rooms)], SizeOf(Rooms[High(Rooms)]), 0);
  Rooms[High(Rooms)].Decription := Desc;
  
  WriteLn('Room added, press enter');
  ReadLn;
end;

// Brisanje sobe
procedure RemoveRoom(RoomIndex: Integer);
var
  I: Integer;
  Door: Integer;
begin
  ClrScr;
  WriteLn('Remove room');
  WriteLn;

  // Ako je RoomIndex van dozvoljenog opsega ne radi nista
  if (RoomIndex >= 1) and (RoomIndex <= High(Rooms) + 1) then
  begin
    // Sredi prolaze
    for I := 0 to High(Rooms) do
    begin
      for Door := CNorth to CEast do
      begin
        // Ako je neka soba vodila u ovu sobu, sredi vrata
        if Rooms[I].Doors[Door] = RoomIndex - 1 then
          Rooms[I].Doors[Door] := -1;
          
        // Sve sobe se pomeraju za jedan index blize nuli
        // Sredi prolaze
        if Rooms[I].Doors[Door] > RoomIndex then
          Rooms[I].Doors[Door] := Rooms[I].Doors[Door] - 1;
      end;
    end;
    
    // Pomeri sve sobe
    for I := RoomIndex to High(Rooms) do
      Rooms[I - 1] := Rooms[I];
    SetLength(Rooms, High(Rooms));
    
    WriteLn('Room deleted');
  end
  else
    WriteLn('Wrong room selected');

  ReadLn;
end;

// Menjanje sobe
procedure EditRoom(RoomIndex: Integer);
var
  YesNo: Char;
  Desc: String;
  Line: String;
begin
  ClrScr;
  WriteLn('Edit room');
  WriteLn;

  // Ako je RoomIndex van dozvoljenog opsega ne radi nista
  if (RoomIndex >= 1) and (RoomIndex <= High(Rooms) + 1) then
  begin
    // Menjanje opisa
    Write('Change description (Y/N): ');
    ReadLn(YesNo);
    if UpCase(YesNo) = 'Y' then
    begin
      WriteLn('Enter description (to finish type <end> at beginig of line):');

      // Uzimamo opis
      Desc := '';
      Line := '';
      repeat
        Desc := #10 + Line;
        ReadLn(Line);
      until Line = '<end>';

      // Izbacujemo prvi #10
      Desc := Copy(Desc, 2, Length(Desc));
      
      Rooms[RoomIndex - 1].Decription := Desc;
    end;

    // Menjanje vrata
    Write('Change north (Y/N): ');
    ReadLn(YesNo);
    if UpCase(YesNo) = 'Y' then
    begin
      Rooms[RoomIndex - 1].Doors[CNorth] := SelectRoom;
    end;
    Write('Change west (Y/N): ');
    ReadLn(YesNo);
    if UpCase(YesNo) = 'Y' then
    begin
      Rooms[RoomIndex - 1].Doors[CWest] := SelectRoom;
    end;
    Write('Change south (Y/N): ');
    ReadLn(YesNo);
    if UpCase(YesNo) = 'Y' then
    begin
      Rooms[RoomIndex - 1].Doors[CSouth] := SelectRoom;
    end;
    Write('Change east (Y/N): ');
    ReadLn(YesNo);
    if UpCase(YesNo) = 'Y' then
    begin
      Rooms[RoomIndex - 1].Doors[CEast] := SelectRoom;
    end;

    WriteLn('Room modified');
  end
  else
    WriteLn('Wrong room selected');

  ReadLn;
end;

// Ucitaj sobe
procedure Load;
var
  FileName: String;
begin
  ClrScr;
  WriteLn('Load rooms');
  WriteLn;
  Write('Enter file name: ');
  ReadLn(FileName);
  if LoadRooms(FileName, Rooms) <> -1 then
    WriteLn('Load complete')
  else
    WriteLn('Error loading rooms');
    
  ReadLn;
end;

// Snimi sobe
procedure Save;
var
  FileName: String;
begin
  ClrScr;
  WriteLn('Save rooms');
  WriteLn;
  Write('Enter file name: ');
  ReadLn(FileName);
  if SaveRooms(FileName, Rooms) <> -1 then
    WriteLn('Save complete')
  else
    WriteLn('Error saving rooms');

  ReadLn;
end;

begin
  SetLength(Rooms, 0);
end.

